package de.hft.stuttgart.citydoctor2.zip;

import de.hft.stuttgart.citydoctor2.datastructure.CityDoctorModel;
import de.hft.stuttgart.citydoctor2.parser.ParserConfiguration;
import org.apache.commons.io.FileUtils;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.ExpectedException;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.ArrayList;
import java.util.List;

import static org.junit.Assert.*;

public class ZipTest {

    ParserConfiguration config = new ParserConfiguration(8,false);

    @Rule
    public ExpectedException excRule = ExpectedException.none();

    @Test
    public void testUnzipping() {
        CityGmlZipArchive cgmlArch = CityGmlZipArchive.register("src/test/resources/zip/mock_archive.zip");
        assertNotNull(cgmlArch);
        cgmlArch.mountArchive(config);
        checkMockArchive(cgmlArch);
    }

    private void checkMockArchive(CityGmlZipArchive cgmlArch){
        assertEquals(5,cgmlArch.getEntries().size());
        for (CityGmlZipEntry entry : cgmlArch.getEntries()) {
            assertNotNull(entry);
            assertTrue(entry.getFileName().matches("^mock[1-5].gml$"));
            assertNull(entry.getErrorType());
            assertNotNull(entry.getModel());
            assertEquals(1, entry.getModel().getNumberOfFeatures());
        }
    }

    @Test
    public void testZipping() throws IOException {
        CityGmlZipArchive cgmlArch = CityGmlZipArchive.register("src/test/resources/zip/mock_archive.zip");
        assertNotNull(cgmlArch);
        cgmlArch.mountArchive(config);
        Path tmpDir = null;
        try {
            tmpDir = Files.createTempDirectory("testTmp");
            tmpDir.toFile().deleteOnExit();
            String expPath = tmpDir.toString() + "/export.zip";
            cgmlArch.exportToZipFile(expPath);
            CityGmlZipArchive cgmlExport = CityGmlZipArchive.register(expPath);
            assertNotNull(cgmlExport);
            cgmlExport.mountArchive(config);
            checkMockArchive(cgmlExport);
        } finally {
            if (tmpDir != null) {
                FileUtils.deleteDirectory(tmpDir.toFile());
            }
        }
    }

    @Test
    public void testEpsgParsing() {
        CityGmlZipArchive cgmlArch = CityGmlZipArchive.register("src/test/resources/zip/epsg.zip");
        assertNotNull(cgmlArch);
        cgmlArch.mountArchive(config);
        List<String> epsgs = new ArrayList<>(2);
        epsgs.add("25832");
        epsgs.add("7415");
        for (CityGmlZipEntry entry : cgmlArch.getEntries()) {
            String srsName = entry.getModel().getCityModel().getBoundedBy().getEnvelope().getSrsName();
            srsName = srsName.split(":")[6];
            assertTrue(epsgs.remove(srsName));
        }
        assertTrue(epsgs.isEmpty());
    }

    @Test
    public void testValidation(){
        ParserConfiguration valConfig = new ParserConfiguration(8,true);
        CityGmlZipArchive cgmlArch = CityGmlZipArchive.register("src/test/resources/zip/validate.zip");
        assertNotNull(cgmlArch);
        cgmlArch.mountArchive(valConfig);
        assertNull(cgmlArch.getEntry("valCorrect.gml").getErrorType());
        assertEquals(ZipEntryErrorType.INVALID_CITY_GML_FILE, cgmlArch.getEntry("valFaulty.gml").getErrorType());
    }

    @Test
    public void testImplicitParsing(){
        CityGmlZipArchive cgmlArch = CityGmlZipArchive.register("src/test/resources/zip/implicit.zip");
        assertNotNull(cgmlArch);
        cgmlArch.mountArchive(config);
        CityDoctorModel mainModel = cgmlArch.getEntry("Main_model.gml").getModel();
        assertEquals(18, mainModel.getGenericCityObjects().size());
    }

}
